@@ -14,8 +14,10 @@
 """Convenience wrapper for invoking APIs/factories w/ a project."""
 
 import os
+import warnings
 
 import google.api_core.client_options
+from google.auth.credentials import AnonymousCredentials
 from google.cloud._helpers import _LocalStack
 from google.cloud._helpers import _determine_default_project as _base_default_project
 from google.cloud.client import ClientWithProject
@@ -27,9 +29,6 @@ from google.cloud.datastore.entity import Entity
 from google.cloud.datastore.key import Key
 from google.cloud.datastore.query import Query
 from google.cloud.datastore.transaction import Transaction
-from google.cloud.environment_vars import DISABLE_GRPC
-from google.cloud.environment_vars import GCD_DATASET
-from google.cloud.environment_vars import GCD_HOST
 
 try:
     from google.cloud.datastore._gapic import make_datastore_api
@@ -54,13 +53,20 @@ _MAX_LOOPS = 128
 _DATASTORE_BASE_URL = "https://datastore.googleapis.com"
 """Datastore API request URL base."""
 
+DATASTORE_EMULATOR_HOST = "DATASTORE_EMULATOR_HOST"
+"""Environment variable defining host for datastore emulator server."""
+DATASTORE_DATASET = "DATASTORE_DATASET"
+"""Environment variable defining default dataset ID under GCD."""
+DISABLE_GRPC = "GOOGLE_CLOUD_DISABLE_GRPC"
+"""Environment variable acting as flag to disable gRPC."""
+
 
 _USE_GRPC = _HAVE_GRPC and not os.getenv(DISABLE_GRPC, False)
 
 
 def _get_gcd_project():
     """Gets the GCD application ID if it can be inferred."""
-    return os.getenv(GCD_DATASET)
+    return os.getenv(DATASTORE_DATASET)
 
 
 def _determine_default_project(project=None):
@@ -89,6 +95,19 @@ def _determine_default_project(project=None):
     return project
 
 
+def _make_retry_timeout_kwargs(retry, timeout):
+    """Helper: make optional retry / timeout kwargs dict."""
+    kwargs = {}
+
+    if retry is not None:
+        kwargs["retry"] = retry
+
+    if timeout is not None:
+        kwargs["timeout"] = timeout
+
+    return kwargs
+
+
 def _extended_lookup(
     datastore_api,
     project,
@@ -97,6 +116,8 @@ def _extended_lookup(
     deferred=None,
     eventual=False,
     transaction_id=None,
+    retry=None,
+    timeout=None,
 ):
     """Repeat lookup until all keys found (unless stop requested).
 
@@ -133,6 +154,17 @@ def _extended_lookup(
                            the given transaction.  Incompatible with
                            ``eventual==True``.
 
+    :type retry: :class:`google.api_core.retry.Retry`
+    :param retry:
+        A retry object used to retry requests. If ``None`` is specified,
+        requests will be retried using a default configuration.
+
+    :type timeout: float
+    :param timeout:
+        Time, in seconds, to wait for the request to complete.
+        Note that if ``retry`` is specified, the timeout applies
+        to each individual attempt.
+
     :rtype: list of :class:`.entity_pb2.Entity`
     :returns: The requested entities.
     :raises: :class:`ValueError` if missing / deferred are not null or
@@ -144,6 +176,8 @@ def _extended_lookup(
     if deferred is not None and deferred != []:
         raise ValueError("deferred must be None or an empty list")
 
+    kwargs = _make_retry_timeout_kwargs(retry, timeout)
+
     results = []
 
     loop_num = 0
@@ -151,7 +185,7 @@ def _extended_lookup(
     while loop_num < _MAX_LOOPS:  # loop against possible deferred.
         loop_num += 1
         lookup_response = datastore_api.lookup(
-            project, key_pbs, read_options=read_options
+            project, key_pbs, read_options=read_options, **kwargs
         )
 
         # Accumulate the new results.
@@ -238,6 +272,15 @@ class Client(ClientWithProject):
         _http=None,
         _use_grpc=None,
     ):
+        emulator_host = os.getenv(DATASTORE_EMULATOR_HOST)
+
+        if emulator_host is not None:
+            if credentials is not None:
+                raise ValueError(
+                    "Explicit credentials are incompatible with the emulator"
+                )
+            credentials = AnonymousCredentials()
+
         super(Client, self).__init__(
             project=project,
             credentials=credentials,
@@ -249,14 +292,15 @@ class Client(ClientWithProject):
         self._client_options = client_options
         self._batch_stack = _LocalStack()
         self._datastore_api_internal = None
+
         if _use_grpc is None:
             self._use_grpc = _USE_GRPC
         else:
             self._use_grpc = _use_grpc
-        try:
-            host = os.environ[GCD_HOST]
-            self._base_url = "http://" + host
-        except KeyError:
+
+        if emulator_host is not None:
+            self._base_url = "http://" + emulator_host
+        else:
             api_endpoint = _DATASTORE_BASE_URL
             if client_options:
                 if type(client_options) == dict:
@@ -338,7 +382,16 @@ class Client(ClientWithProject):
         if isinstance(transaction, Transaction):
             return transaction
 
-    def get(self, key, missing=None, deferred=None, transaction=None, eventual=False):
+    def get(
+        self,
+        key,
+        missing=None,
+        deferred=None,
+        transaction=None,
+        eventual=False,
+        retry=None,
+        timeout=None,
+    ):
         """Retrieve an entity from a single key (if it exists).
 
         .. note::
@@ -369,6 +422,17 @@ class Client(ClientWithProject):
                          Setting True will use eventual consistency, but cannot
                          be used inside a transaction or will raise ValueError.
 
+        :type retry: :class:`google.api_core.retry.Retry`
+        :param retry:
+            A retry object used to retry requests. If ``None`` is specified,
+            requests will be retried using a default configuration.
+
+        :type timeout: float
+        :param timeout:
+            Time, in seconds, to wait for the request to complete.
+            Note that if ``retry`` is specified, the timeout applies
+            to each individual attempt.
+
         :rtype: :class:`google.cloud.datastore.entity.Entity` or ``NoneType``
         :returns: The requested entity if it exists.
 
@@ -380,12 +444,21 @@ class Client(ClientWithProject):
             deferred=deferred,
             transaction=transaction,
             eventual=eventual,
+            retry=retry,
+            timeout=timeout,
         )
         if entities:
             return entities[0]
 
     def get_multi(
-        self, keys, missing=None, deferred=None, transaction=None, eventual=False
+        self,
+        keys,
+        missing=None,
+        deferred=None,
+        transaction=None,
+        eventual=False,
+        retry=None,
+        timeout=None,
     ):
         """Retrieve entities, along with their attributes.
 
@@ -412,6 +485,17 @@ class Client(ClientWithProject):
                          Setting True will use eventual consistency, but cannot
                          be used inside a transaction or will raise ValueError.
 
+        :type retry: :class:`google.api_core.retry.Retry`
+        :param retry:
+            A retry object used to retry requests. If ``None`` is specified,
+            requests will be retried using a default configuration.
+
+        :type timeout: float
+        :param timeout:
+            Time, in seconds, to wait for the request to complete.
+            Note that if ``retry`` is specified, the timeout applies
+            to each individual attempt.
+
         :rtype: list of :class:`google.cloud.datastore.entity.Entity`
         :returns: The requested entities.
         :raises: :class:`ValueError` if one or more of ``keys`` has a project
@@ -437,6 +521,8 @@ class Client(ClientWithProject):
             missing=missing,
             deferred=deferred,
             transaction_id=transaction and transaction.id,
+            retry=retry,
+            timeout=timeout,
         )
 
         if missing is not None:
@@ -451,7 +537,7 @@ class Client(ClientWithProject):
 
         return [helpers.entity_from_protobuf(entity_pb) for entity_pb in entity_pbs]
 
-    def put(self, entity):
+    def put(self, entity, retry=None, timeout=None):
         """Save an entity in the Cloud Datastore.
 
         .. note::
@@ -462,15 +548,41 @@ class Client(ClientWithProject):
 
         :type entity: :class:`google.cloud.datastore.entity.Entity`
         :param entity: The entity to be saved to the datastore.
-        """
-        self.put_multi(entities=[entity])
 
-    def put_multi(self, entities):
+        :type retry: :class:`google.api_core.retry.Retry`
+        :param retry:
+            A retry object used to retry requests. If ``None`` is specified,
+            requests will be retried using a default configuration.
+            Only meaningful outside of another batch / transaction.
+
+        :type timeout: float
+        :param timeout:
+            Time, in seconds, to wait for the request to complete.
+            Note that if ``retry`` is specified, the timeout applies
+            to each individual attempt.  Only meaningful outside of another
+            batch / transaction.
+        """
+        self.put_multi(entities=[entity], retry=retry, timeout=timeout)
+
+    def put_multi(self, entities, retry=None, timeout=None):
         """Save entities in the Cloud Datastore.
 
         :type entities: list of :class:`google.cloud.datastore.entity.Entity`
         :param entities: The entities to be saved to the datastore.
 
+        :type retry: :class:`google.api_core.retry.Retry`
+        :param retry:
+            A retry object used to retry requests. If ``None`` is specified,
+            requests will be retried using a default configuration.
+            Only meaningful outside of another batch / transaction.
+
+        :type timeout: float
+        :param timeout:
+            Time, in seconds, to wait for the request to complete.
+            Note that if ``retry`` is specified, the timeout applies
+            to each individual attempt.  Only meaningful outside of another
+            batch / transaction.
+
         :raises: :class:`ValueError` if ``entities`` is a single entity.
         """
         if isinstance(entities, Entity):
@@ -490,9 +602,9 @@ class Client(ClientWithProject):
             current.put(entity)
 
         if not in_batch:
-            current.commit()
+            current.commit(retry=retry, timeout=timeout)
 
-    def delete(self, key):
+    def delete(self, key, retry=None, timeout=None):
         """Delete the key in the Cloud Datastore.
 
         .. note::
@@ -503,14 +615,40 @@ class Client(ClientWithProject):
 
         :type key: :class:`google.cloud.datastore.key.Key`
         :param key: The key to be deleted from the datastore.
-        """
-        self.delete_multi(keys=[key])
 
-    def delete_multi(self, keys):
+        :type retry: :class:`google.api_core.retry.Retry`
+        :param retry:
+            A retry object used to retry requests. If ``None`` is specified,
+            requests will be retried using a default configuration.
+            Only meaningful outside of another batch / transaction.
+
+        :type timeout: float
+        :param timeout:
+            Time, in seconds, to wait for the request to complete.
+            Note that if ``retry`` is specified, the timeout applies
+            to each individual attempt.  Only meaningful outside of another
+            batch / transaction.
+        """
+        self.delete_multi(keys=[key], retry=retry, timeout=timeout)
+
+    def delete_multi(self, keys, retry=None, timeout=None):
         """Delete keys from the Cloud Datastore.
 
         :type keys: list of :class:`google.cloud.datastore.key.Key`
         :param keys: The keys to be deleted from the Datastore.
+
+        :type retry: :class:`google.api_core.retry.Retry`
+        :param retry:
+            A retry object used to retry requests. If ``None`` is specified,
+            requests will be retried using a default configuration.
+            Only meaningful outside of another batch / transaction.
+
+        :type timeout: float
+        :param timeout:
+            Time, in seconds, to wait for the request to complete.
+            Note that if ``retry`` is specified, the timeout applies
+            to each individual attempt.  Only meaningful outside of another
+            batch / transaction.
         """
         if not keys:
             return
@@ -527,9 +665,9 @@ class Client(ClientWithProject):
             current.delete(key)
 
         if not in_batch:
-            current.commit()
+            current.commit(retry=retry, timeout=timeout)
 
-    def allocate_ids(self, incomplete_key, num_ids):
+    def allocate_ids(self, incomplete_key, num_ids, retry=None, timeout=None):
         """Allocate a list of IDs from a partial key.
 
         :type incomplete_key: :class:`google.cloud.datastore.key.Key`
@@ -538,6 +676,17 @@ class Client(ClientWithProject):
         :type num_ids: int
         :param num_ids: The number of IDs to allocate.
 
+        :type retry: :class:`google.api_core.retry.Retry`
+        :param retry:
+            A retry object used to retry requests. If ``None`` is specified,
+            requests will be retried using a default configuration.
+
+        :type timeout: float
+        :param timeout:
+            Time, in seconds, to wait for the request to complete.
+            Note that if ``retry`` is specified, the timeout applies
+            to each individual attempt.
+
         :rtype: list of :class:`google.cloud.datastore.key.Key`
         :returns: The (complete) keys allocated with ``incomplete_key`` as
                   root.
@@ -550,8 +699,10 @@ class Client(ClientWithProject):
         incomplete_key_pb = incomplete_key.to_protobuf()
         incomplete_key_pbs = [incomplete_key_pb] * num_ids
 
+        kwargs = _make_retry_timeout_kwargs(retry, timeout)
+
         response_pb = self._datastore_api.allocate_ids(
-            incomplete_key.project, incomplete_key_pbs
+            incomplete_key.project, incomplete_key_pbs, **kwargs
         )
         allocated_ids = [
             allocated_key_pb.path[-1].id for allocated_key_pb in response_pb.keys
@@ -666,15 +817,33 @@ class Client(ClientWithProject):
             kwargs["namespace"] = self.namespace
         return Query(self, **kwargs)
 
-    def reserve_ids(self, complete_key, num_ids):
-        """Reserve a list of IDs from a complete key.
+    def reserve_ids_sequential(self, complete_key, num_ids, retry=None, timeout=None):
+        """Reserve a list of IDs sequentially from a complete key.
+
+        This will reserve the key passed as `complete_key` as well as
+        additional keys derived by incrementing the last ID in the path of
+        `complete_key` sequentially to obtain the number of keys specified in
+        `num_ids`.
 
         :type complete_key: :class:`google.cloud.datastore.key.Key`
-        :param complete_key: Complete key to use as base for reserved IDs.
+        :param complete_key:
+            Complete key to use as base for reserved IDs. Key must use a
+            numeric ID and not a string name.
 
         :type num_ids: int
         :param num_ids: The number of IDs to reserve.
 
+        :type retry: :class:`google.api_core.retry.Retry`
+        :param retry:
+            A retry object used to retry requests. If ``None`` is specified,
+            requests will be retried using a default configuration.
+
+        :type timeout: float
+        :param timeout:
+            Time, in seconds, to wait for the request to complete.
+            Note that if ``retry`` is specified, the timeout applies
+            to each individual attempt.
+
         :rtype: class:`NoneType`
         :returns: None
         :raises: :class:`ValueError` if `complete_key`` is not a
@@ -683,12 +852,75 @@ class Client(ClientWithProject):
         if complete_key.is_partial:
             raise ValueError(("Key is not Complete.", complete_key))
 
+        if complete_key.id is None:
+            raise ValueError(("Key must use numeric id.", complete_key))
+
         if not isinstance(num_ids, int):
             raise ValueError(("num_ids is not a valid integer.", num_ids))
 
-        complete_key_pb = complete_key.to_protobuf()
-        complete_key_pbs = [complete_key_pb] * num_ids
+        key_class = type(complete_key)
+        namespace = complete_key._namespace
+        project = complete_key._project
+        flat_path = list(complete_key._flat_path[:-1])
+        start_id = complete_key._flat_path[-1]
 
-        self._datastore_api.reserve_ids(complete_key.project, complete_key_pbs)
+        key_pbs = []
+        for id in range(start_id, start_id + num_ids):
+            path = flat_path + [id]
+            key = key_class(*path, project=project, namespace=namespace)
+            key_pbs.append(key.to_protobuf())
+
+        kwargs = _make_retry_timeout_kwargs(retry, timeout)
+        self._datastore_api.reserve_ids(complete_key.project, key_pbs, **kwargs)
+
+        return None
+
+    def reserve_ids(self, complete_key, num_ids, retry=None, timeout=None):
+        """Reserve a list of IDs sequentially from a complete key.
+
+        DEPRECATED. Alias for :meth:`reserve_ids_sequential`.
+
+        Please use either :meth:`reserve_ids_multi` (recommended) or
+        :meth:`reserve_ids_sequential`.
+        """
+        message = (
+            "Client.reserve_ids is deprecated. Please use "
+            "Client.reserve_ids_multi or Client.reserve_ids_sequential",
+        )
+        warnings.warn(message, DeprecationWarning)
+        return self.reserve_ids_sequential(
+            complete_key, num_ids, retry=retry, timeout=timeout
+        )
+
+    def reserve_ids_multi(self, complete_keys, retry=None, timeout=None):
+        """Reserve IDs from a list of complete keys.
+
+        :type complete_keys: `list` of :class:`google.cloud.datastore.key.Key`
+        :param complete_keys:
+            Complete keys for which to reserve IDs.
+
+        :type retry: :class:`google.api_core.retry.Retry`
+        :param retry:
+            A retry object used to retry requests. If ``None`` is specified,
+            requests will be retried using a default configuration.
+
+        :type timeout: float
+        :param timeout:
+            Time, in seconds, to wait for the request to complete.
+            Note that if ``retry`` is specified, the timeout applies
+            to each individual attempt.
+
+        :rtype: class:`NoneType`
+        :returns: None
+        :raises: :class:`ValueError` if any of `complete_keys`` is not a
+                 Complete key.
+        """
+        for complete_key in complete_keys:
+            if complete_key.is_partial:
+                raise ValueError(("Key is not Complete.", complete_key))
+
+        kwargs = _make_retry_timeout_kwargs(retry, timeout)
+        key_pbs = [key.to_protobuf() for key in complete_keys]
+        self._datastore_api.reserve_ids(complete_keys[0].project, key_pbs, **kwargs)
 
         return None
