@@ -52,10 +52,10 @@ class Test__get_gcd_project(unittest.TestCase):
             self.assertIsNone(project)
 
     def test_value_set(self):
-        from google.cloud.datastore.client import GCD_DATASET
+        from google.cloud.datastore.client import DATASTORE_DATASET
 
         MOCK_PROJECT = object()
-        environ = {GCD_DATASET: MOCK_PROJECT}
+        environ = {DATASTORE_DATASET: MOCK_PROJECT}
         with mock.patch("os.getenv", new=environ.get):
             project = self._call_fut()
             self.assertEqual(project, MOCK_PROJECT)
@@ -235,18 +235,33 @@ class TestClient(unittest.TestCase):
             )
             self.assertTrue(client4._use_grpc)
 
-    def test_constructor_gcd_host(self):
-        from google.cloud.environment_vars import GCD_HOST
+    def test_constructor_w_emulator_w_creds(self):
+        from google.cloud.datastore.client import DATASTORE_EMULATOR_HOST
 
         host = "localhost:1234"
-        fake_environ = {GCD_HOST: host}
+        fake_environ = {DATASTORE_EMULATOR_HOST: host}
         project = "PROJECT"
         creds = _make_credentials()
         http = object()
 
         with mock.patch("os.environ", new=fake_environ):
-            client = self._make_one(project=project, credentials=creds, _http=http)
-            self.assertEqual(client.base_url, "http://" + host)
+            with self.assertRaises(ValueError):
+                self._make_one(project=project, credentials=creds, _http=http)
+
+    def test_constructor_w_emulator_wo_creds(self):
+        from google.auth.credentials import AnonymousCredentials
+        from google.cloud.datastore.client import DATASTORE_EMULATOR_HOST
+
+        host = "localhost:1234"
+        fake_environ = {DATASTORE_EMULATOR_HOST: host}
+        project = "PROJECT"
+        http = object()
+
+        with mock.patch("os.environ", new=fake_environ):
+            client = self._make_one(project=project, _http=http)
+
+        self.assertEqual(client.base_url, "http://" + host)
+        self.assertIsInstance(client._credentials, AnonymousCredentials)
 
     def test_base_url_property(self):
         from google.cloud.datastore.client import _DATASTORE_BASE_URL
@@ -356,25 +371,24 @@ class TestClient(unittest.TestCase):
         self.assertEqual(list(client._batch_stack), [])
 
     def test_get_miss(self):
-        _called_with = []
-
-        def _get_multi(*args, **kw):
-            _called_with.append((args, kw))
-            return []
 
         creds = _make_credentials()
         client = self._make_one(credentials=creds)
-        client.get_multi = _get_multi
+        get_multi = client.get_multi = mock.Mock(return_value=[])
 
         key = object()
 
         self.assertIsNone(client.get(key))
 
-        self.assertEqual(_called_with[0][0], ())
-        self.assertEqual(_called_with[0][1]["keys"], [key])
-        self.assertIsNone(_called_with[0][1]["missing"])
-        self.assertIsNone(_called_with[0][1]["deferred"])
-        self.assertIsNone(_called_with[0][1]["transaction"])
+        get_multi.assert_called_once_with(
+            keys=[key],
+            missing=None,
+            deferred=None,
+            transaction=None,
+            eventual=False,
+            retry=None,
+            timeout=None,
+        )
 
     def test_get_hit(self):
         TXN_ID = "123"
@@ -554,13 +568,15 @@ class TestClient(unittest.TestCase):
             self.PROJECT, [key1_pb, key2_pb], read_options=read_options
         )
 
-    def test_get_multi_hit(self):
+    def test_get_multi_hit_w_retry_w_timeout(self):
         from google.cloud.datastore_v1.proto import datastore_pb2
         from google.cloud.datastore.key import Key
 
         kind = "Kind"
         id_ = 1234
         path = [{"kind": kind, "id": id_}]
+        retry = mock.Mock()
+        timeout = 100000
 
         # Make a found entity pb to be returned from mock backend.
         entity_pb = _make_entity_pb(self.PROJECT, kind, id_, "foo", "Foo")
@@ -573,7 +589,7 @@ class TestClient(unittest.TestCase):
         client._datastore_api_internal = ds_api
 
         key = Key(kind, id_, project=self.PROJECT)
-        (result,) = client.get_multi([key])
+        (result,) = client.get_multi([key], retry=retry, timeout=timeout)
         new_key = result.key
 
         # Check the returned value is as expected.
@@ -585,7 +601,11 @@ class TestClient(unittest.TestCase):
 
         read_options = datastore_pb2.ReadOptions()
         ds_api.lookup.assert_called_once_with(
-            self.PROJECT, [key.to_protobuf()], read_options=read_options
+            self.PROJECT,
+            [key.to_protobuf()],
+            read_options=read_options,
+            retry=retry,
+            timeout=timeout,
         )
 
     def test_get_multi_hit_w_transaction(self):
@@ -711,20 +731,30 @@ class TestClient(unittest.TestCase):
         ds_api.lookup.assert_not_called()
 
     def test_put(self):
-        _called_with = []
-
-        def _put_multi(*args, **kw):
-            _called_with.append((args, kw))
 
         creds = _make_credentials()
         client = self._make_one(credentials=creds)
-        client.put_multi = _put_multi
-        entity = object()
+        put_multi = client.put_multi = mock.Mock()
+        entity = mock.Mock()
 
         client.put(entity)
 
-        self.assertEqual(_called_with[0][0], ())
-        self.assertEqual(_called_with[0][1]["entities"], [entity])
+        put_multi.assert_called_once_with(entities=[entity], retry=None, timeout=None)
+
+    def test_put_w_retry_w_timeout(self):
+
+        creds = _make_credentials()
+        client = self._make_one(credentials=creds)
+        put_multi = client.put_multi = mock.Mock()
+        entity = mock.Mock()
+        retry = mock.Mock()
+        timeout = 100000
+
+        client.put(entity, retry=retry, timeout=timeout)
+
+        put_multi.assert_called_once_with(
+            entities=[entity], retry=retry, timeout=timeout
+        )
 
     def test_put_multi_no_entities(self):
         creds = _make_credentials()
@@ -739,13 +769,14 @@ class TestClient(unittest.TestCase):
         client = self._make_one(credentials=creds)
         self.assertRaises(ValueError, client.put_multi, Entity())
 
-    def test_put_multi_no_batch_w_partial_key(self):
+    def test_put_multi_no_batch_w_partial_key_w_retry_w_timeout(self):
         from google.cloud.datastore_v1.proto import datastore_pb2
         from google.cloud.datastore.helpers import _property_tuples
 
         entity = _Entity(foo=u"bar")
-        key = entity.key = _Key(self.PROJECT)
-        key._id = None
+        key = entity.key = _Key(_Key.kind, None)
+        retry = mock.Mock()
+        timeout = 100000
 
         creds = _make_credentials()
         client = self._make_one(credentials=creds)
@@ -753,12 +784,13 @@ class TestClient(unittest.TestCase):
         ds_api = _make_datastore_api(key_pb)
         client._datastore_api_internal = ds_api
 
-        result = client.put_multi([entity])
+        result = client.put_multi([entity], retry=retry, timeout=timeout)
         self.assertIsNone(result)
 
         self.assertEqual(ds_api.commit.call_count, 1)
         _, positional, keyword = ds_api.commit.mock_calls[0]
-        self.assertEqual(keyword, {"transaction": None})
+        expected_kw = {"transaction": None, "retry": retry, "timeout": timeout}
+        self.assertEqual(keyword, expected_kw)
 
         self.assertEqual(len(positional), 3)
         self.assertEqual(positional[0], self.PROJECT)
@@ -780,7 +812,7 @@ class TestClient(unittest.TestCase):
         creds = _make_credentials()
         client = self._make_one(credentials=creds)
         entity = _Entity(foo=u"bar")
-        key = entity.key = _Key(self.PROJECT)
+        key = entity.key = _Key()
 
         with _NoCommitBatch(client) as CURR_BATCH:
             result = client.put_multi([entity])
@@ -796,20 +828,26 @@ class TestClient(unittest.TestCase):
         self.assertEqual(value_pb.string_value, u"bar")
 
     def test_delete(self):
-        _called_with = []
-
-        def _delete_multi(*args, **kw):
-            _called_with.append((args, kw))
-
         creds = _make_credentials()
         client = self._make_one(credentials=creds)
-        client.delete_multi = _delete_multi
-        key = object()
+        delete_multi = client.delete_multi = mock.Mock()
+        key = mock.Mock()
 
         client.delete(key)
 
-        self.assertEqual(_called_with[0][0], ())
-        self.assertEqual(_called_with[0][1]["keys"], [key])
+        delete_multi.assert_called_once_with(keys=[key], retry=None, timeout=None)
+
+    def test_delete_w_retry_w_timeout(self):
+        creds = _make_credentials()
+        client = self._make_one(credentials=creds)
+        delete_multi = client.delete_multi = mock.Mock()
+        key = mock.Mock()
+        retry = mock.Mock()
+        timeout = 100000
+
+        client.delete(key, retry=retry, timeout=timeout)
+
+        delete_multi.assert_called_once_with(keys=[key], retry=retry, timeout=timeout)
 
     def test_delete_multi_no_keys(self):
         creds = _make_credentials()
@@ -820,22 +858,25 @@ class TestClient(unittest.TestCase):
         self.assertIsNone(result)
         client._datastore_api_internal.commit.assert_not_called()
 
-    def test_delete_multi_no_batch(self):
+    def test_delete_multi_no_batch_w_retry_w_timeout(self):
         from google.cloud.datastore_v1.proto import datastore_pb2
 
-        key = _Key(self.PROJECT)
+        key = _Key()
+        retry = mock.Mock()
+        timeout = 100000
 
         creds = _make_credentials()
         client = self._make_one(credentials=creds)
         ds_api = _make_datastore_api()
         client._datastore_api_internal = ds_api
 
-        result = client.delete_multi([key])
+        result = client.delete_multi([key], retry=retry, timeout=timeout)
         self.assertIsNone(result)
 
         self.assertEqual(ds_api.commit.call_count, 1)
         _, positional, keyword = ds_api.commit.mock_calls[0]
-        self.assertEqual(keyword, {"transaction": None})
+        expected_kw = {"transaction": None, "retry": retry, "timeout": timeout}
+        self.assertEqual(keyword, expected_kw)
 
         self.assertEqual(len(positional), 3)
         self.assertEqual(positional[0], self.PROJECT)
@@ -850,7 +891,7 @@ class TestClient(unittest.TestCase):
         client = self._make_one(credentials=creds)
         client._datastore_api_internal = _make_datastore_api()
 
-        key = _Key(self.PROJECT)
+        key = _Key()
 
         with _NoCommitBatch(client) as CURR_BATCH:
             result = client.delete_multi([key])
@@ -865,7 +906,7 @@ class TestClient(unittest.TestCase):
         client = self._make_one(credentials=creds)
         client._datastore_api_internal = _make_datastore_api()
 
-        key = _Key(self.PROJECT)
+        key = _Key()
 
         with _NoCommitTransaction(client) as CURR_XACT:
             result = client.delete_multi([key])
@@ -878,8 +919,7 @@ class TestClient(unittest.TestCase):
     def test_allocate_ids_w_partial_key(self):
         num_ids = 2
 
-        incomplete_key = _Key(self.PROJECT)
-        incomplete_key._id = None
+        incomplete_key = _Key(_Key.kind, None)
 
         creds = _make_credentials()
         client = self._make_one(credentials=creds, _use_grpc=False)
@@ -891,32 +931,197 @@ class TestClient(unittest.TestCase):
         result = client.allocate_ids(incomplete_key, num_ids)
 
         # Check the IDs returned.
-        self.assertEqual([key._id for key in result], list(range(num_ids)))
+        self.assertEqual([key.id for key in result], list(range(num_ids)))
+
+        expected_keys = [incomplete_key.to_protobuf()] * num_ids
+        alloc_ids.assert_called_once_with(self.PROJECT, expected_keys)
+
+    def test_allocate_ids_w_partial_key_w_retry_w_timeout(self):
+        num_ids = 2
+
+        incomplete_key = _Key(_Key.kind, None)
+        retry = mock.Mock()
+        timeout = 100000
+
+        creds = _make_credentials()
+        client = self._make_one(credentials=creds, _use_grpc=False)
+        allocated = mock.Mock(keys=[_KeyPB(i) for i in range(num_ids)], spec=["keys"])
+        alloc_ids = mock.Mock(return_value=allocated, spec=[])
+        ds_api = mock.Mock(allocate_ids=alloc_ids, spec=["allocate_ids"])
+        client._datastore_api_internal = ds_api
+
+        result = client.allocate_ids(
+            incomplete_key, num_ids, retry=retry, timeout=timeout
+        )
+
+        # Check the IDs returned.
+        self.assertEqual([key.id for key in result], list(range(num_ids)))
+
+        expected_keys = [incomplete_key.to_protobuf()] * num_ids
+        alloc_ids.assert_called_once_with(
+            self.PROJECT, expected_keys, retry=retry, timeout=timeout
+        )
 
     def test_allocate_ids_w_completed_key(self):
         creds = _make_credentials()
         client = self._make_one(credentials=creds)
 
-        complete_key = _Key(self.PROJECT)
+        complete_key = _Key()
         self.assertRaises(ValueError, client.allocate_ids, complete_key, 2)
 
+    def test_reserve_ids_sequential_w_completed_key(self):
+        num_ids = 2
+        creds = _make_credentials()
+        client = self._make_one(credentials=creds, _use_grpc=False)
+        complete_key = _Key()
+        reserve_ids = mock.Mock()
+        ds_api = mock.Mock(reserve_ids=reserve_ids, spec=["reserve_ids"])
+        client._datastore_api_internal = ds_api
+        self.assertTrue(not complete_key.is_partial)
+
+        client.reserve_ids_sequential(complete_key, num_ids)
+
+        reserved_keys = (
+            _Key(_Key.kind, id)
+            for id in range(complete_key.id, complete_key.id + num_ids)
+        )
+        expected_keys = [key.to_protobuf() for key in reserved_keys]
+        reserve_ids.assert_called_once_with(self.PROJECT, expected_keys)
+
+    def test_reserve_ids_sequential_w_completed_key_w_retry_w_timeout(self):
+        num_ids = 2
+        retry = mock.Mock()
+        timeout = 100000
+
+        creds = _make_credentials()
+        client = self._make_one(credentials=creds, _use_grpc=False)
+        complete_key = _Key()
+        self.assertTrue(not complete_key.is_partial)
+        reserve_ids = mock.Mock()
+        ds_api = mock.Mock(reserve_ids=reserve_ids, spec=["reserve_ids"])
+        client._datastore_api_internal = ds_api
+
+        client.reserve_ids_sequential(
+            complete_key, num_ids, retry=retry, timeout=timeout
+        )
+
+        reserved_keys = (
+            _Key(_Key.kind, id)
+            for id in range(complete_key.id, complete_key.id + num_ids)
+        )
+        expected_keys = [key.to_protobuf() for key in reserved_keys]
+        reserve_ids.assert_called_once_with(
+            self.PROJECT, expected_keys, retry=retry, timeout=timeout
+        )
+
+    def test_reserve_ids_sequential_w_completed_key_w_ancestor(self):
+        num_ids = 2
+        creds = _make_credentials()
+        client = self._make_one(credentials=creds, _use_grpc=False)
+        complete_key = _Key("PARENT", "SINGLETON", _Key.kind, 1234)
+        reserve_ids = mock.Mock()
+        ds_api = mock.Mock(reserve_ids=reserve_ids, spec=["reserve_ids"])
+        client._datastore_api_internal = ds_api
+        self.assertTrue(not complete_key.is_partial)
+
+        client.reserve_ids_sequential(complete_key, num_ids)
+
+        reserved_keys = (
+            _Key("PARENT", "SINGLETON", _Key.kind, id)
+            for id in range(complete_key.id, complete_key.id + num_ids)
+        )
+        expected_keys = [key.to_protobuf() for key in reserved_keys]
+        reserve_ids.assert_called_once_with(self.PROJECT, expected_keys)
+
+    def test_reserve_ids_sequential_w_partial_key(self):
+        num_ids = 2
+        incomplete_key = _Key(_Key.kind, None)
+        creds = _make_credentials()
+        client = self._make_one(credentials=creds)
+        with self.assertRaises(ValueError):
+            client.reserve_ids_sequential(incomplete_key, num_ids)
+
+    def test_reserve_ids_sequential_w_wrong_num_ids(self):
+        num_ids = "2"
+        complete_key = _Key()
+        creds = _make_credentials()
+        client = self._make_one(credentials=creds)
+        with self.assertRaises(ValueError):
+            client.reserve_ids_sequential(complete_key, num_ids)
+
+    def test_reserve_ids_sequential_w_non_numeric_key_name(self):
+        num_ids = 2
+        complete_key = _Key(_Key.kind, "batman")
+        creds = _make_credentials()
+        client = self._make_one(credentials=creds)
+        with self.assertRaises(ValueError):
+            client.reserve_ids_sequential(complete_key, num_ids)
+
     def test_reserve_ids_w_completed_key(self):
         num_ids = 2
         creds = _make_credentials()
         client = self._make_one(credentials=creds, _use_grpc=False)
-        complete_key = _Key(self.PROJECT)
+        complete_key = _Key()
         reserve_ids = mock.Mock()
         ds_api = mock.Mock(reserve_ids=reserve_ids, spec=["reserve_ids"])
         client._datastore_api_internal = ds_api
         self.assertTrue(not complete_key.is_partial)
+
         client.reserve_ids(complete_key, num_ids)
-        expected_keys = [complete_key.to_protobuf()] * num_ids
+
+        reserved_keys = (
+            _Key(_Key.kind, id)
+            for id in range(complete_key.id, complete_key.id + num_ids)
+        )
+        expected_keys = [key.to_protobuf() for key in reserved_keys]
+        reserve_ids.assert_called_once_with(self.PROJECT, expected_keys)
+
+    def test_reserve_ids_w_completed_key_w_retry_w_timeout(self):
+        num_ids = 2
+        retry = mock.Mock()
+        timeout = 100000
+
+        creds = _make_credentials()
+        client = self._make_one(credentials=creds, _use_grpc=False)
+        complete_key = _Key()
+        self.assertTrue(not complete_key.is_partial)
+        reserve_ids = mock.Mock()
+        ds_api = mock.Mock(reserve_ids=reserve_ids, spec=["reserve_ids"])
+        client._datastore_api_internal = ds_api
+
+        client.reserve_ids(complete_key, num_ids, retry=retry, timeout=timeout)
+
+        reserved_keys = (
+            _Key(_Key.kind, id)
+            for id in range(complete_key.id, complete_key.id + num_ids)
+        )
+        expected_keys = [key.to_protobuf() for key in reserved_keys]
+        reserve_ids.assert_called_once_with(
+            self.PROJECT, expected_keys, retry=retry, timeout=timeout
+        )
+
+    def test_reserve_ids_w_completed_key_w_ancestor(self):
+        num_ids = 2
+        creds = _make_credentials()
+        client = self._make_one(credentials=creds, _use_grpc=False)
+        complete_key = _Key("PARENT", "SINGLETON", _Key.kind, 1234)
+        reserve_ids = mock.Mock()
+        ds_api = mock.Mock(reserve_ids=reserve_ids, spec=["reserve_ids"])
+        client._datastore_api_internal = ds_api
+        self.assertTrue(not complete_key.is_partial)
+
+        client.reserve_ids(complete_key, num_ids)
+
+        reserved_keys = (
+            _Key("PARENT", "SINGLETON", _Key.kind, id)
+            for id in range(complete_key.id, complete_key.id + num_ids)
+        )
+        expected_keys = [key.to_protobuf() for key in reserved_keys]
         reserve_ids.assert_called_once_with(self.PROJECT, expected_keys)
 
     def test_reserve_ids_w_partial_key(self):
         num_ids = 2
-        incomplete_key = _Key(self.PROJECT)
-        incomplete_key._id = None
+        incomplete_key = _Key(_Key.kind, None)
         creds = _make_credentials()
         client = self._make_one(credentials=creds)
         with self.assertRaises(ValueError):
@@ -924,12 +1129,41 @@ class TestClient(unittest.TestCase):
 
     def test_reserve_ids_w_wrong_num_ids(self):
         num_ids = "2"
-        complete_key = _Key(self.PROJECT)
+        complete_key = _Key()
         creds = _make_credentials()
         client = self._make_one(credentials=creds)
         with self.assertRaises(ValueError):
             client.reserve_ids(complete_key, num_ids)
 
+    def test_reserve_ids_w_non_numeric_key_name(self):
+        num_ids = 2
+        complete_key = _Key(_Key.kind, "batman")
+        creds = _make_credentials()
+        client = self._make_one(credentials=creds)
+        with self.assertRaises(ValueError):
+            client.reserve_ids(complete_key, num_ids)
+
+    def test_reserve_ids_multi(self):
+        creds = _make_credentials()
+        client = self._make_one(credentials=creds, _use_grpc=False)
+        key1 = _Key(_Key.kind, "one")
+        key2 = _Key(_Key.kind, "two")
+        reserve_ids = mock.Mock()
+        ds_api = mock.Mock(reserve_ids=reserve_ids, spec=["reserve_ids"])
+        client._datastore_api_internal = ds_api
+
+        client.reserve_ids_multi([key1, key2])
+
+        expected_keys = [key1.to_protobuf(), key2.to_protobuf()]
+        reserve_ids.assert_called_once_with(self.PROJECT, expected_keys)
+
+    def test_reserve_ids_multi_w_partial_key(self):
+        incomplete_key = _Key(_Key.kind, None)
+        creds = _make_credentials()
+        client = self._make_one(credentials=creds)
+        with self.assertRaises(ValueError):
+            client.reserve_ids_multi([incomplete_key])
+
     def test_key_w_project(self):
         KIND = "KIND"
         ID = 1234
@@ -1160,38 +1394,64 @@ class _Entity(dict):
 
 
 class _Key(object):
-    _MARKER = object()
-    _kind = "KIND"
+    kind = "KIND"
+    id = 1234
+    name = None
+    _project = project = "PROJECT"
+    _namespace = None
+
     _key = "KEY"
     _path = None
-    _id = 1234
     _stored = None
 
-    def __init__(self, project):
-        self.project = project
+    def __init__(self, *flat_path, **kwargs):
+        if flat_path:
+            self._flat_path = flat_path
+            self.kind = flat_path[-2]
+            id_or_name = flat_path[-1]
+            if isinstance(id_or_name, int):
+                self.id = id_or_name
+            else:
+                self.id = None
+                self.name = id_or_name
+
+        else:
+            self._flat_path = [self.kind, self.id]
+
+        self.__dict__.update(kwargs)
+        self._kw_args = kwargs
 
     @property
     def is_partial(self):
-        return self._id is None
+        return self.id is None and self.name is None
 
     def to_protobuf(self):
         from google.cloud.datastore_v1.proto import entity_pb2
 
         key = self._key = entity_pb2.Key()
-        # Don't assign it, because it will just get ripped out
-        # key.partition_id.project_id = self.project
 
-        element = key.path.add()
-        element.kind = self._kind
-        if self._id is not None:
-            element.id = self._id
+        path = self._flat_path
+        while path:
+            element = key.path.add()
+            kind, id_or_name = path[:2]
+            element.kind = kind
+            if isinstance(id_or_name, int):
+                element.id = id_or_name
+            elif id_or_name is not None:
+                element.name = id_or_name
+
+            path = path[2:]
 
         return key
 
     def completed_key(self, new_id):
         assert self.is_partial
-        new_key = self.__class__(self.project)
-        new_key._id = new_id
+
+        path = list(self._flat_path)
+        path[-1] = new_id
+
+        key_class = type(self)
+        new_key = key_class(*path, **self._kw_args)
         return new_key
 
 
